﻿<#
.SYNOPSIS
    A script that demonstrates PowerShell integration with REST API.
.DESCRIPTION
    Authenticate and call "Upload or update user embedded certificate for non-dynamics stuff, a password is required, can be empty if it has one" REST API.
.PARAMETER uemHostAndPort
    Host and port of the UEM.
.PARAMETER tenantGuid
    TenantGuid to use for authentication.
.PARAMETER authType
    Authentication type to use.  (Local or AD).
.PARAMETER username
    Username to use for authentication.
.PARAMETER password
    Password to use for authentication.
.PARAMETER domain
    Domain to use for authentication if authType=AD.
.PARAMETER userGuid
    User guid by which the search will be performed.
.PARAMETER certPath
    Path to required certificate.
.PARAMETER certPassword
    Certificate password.
.PARAMETER profileGuid
    Profile guid by which certificate will be updated in a policy.
.EXAMPLE
        upload embedded certificate with password
        .\UploadCertificateWithPassword.ps1 -uemHostAndPort server01:18084 -tenantGuid SRP00000 -authType Local -username admin -password password -domain example -userGuid c4321652-24d9-46b3-803c-e5269bded459 -certPath C:\Users\username\Desktop\certificate.pfx -certPassword password
    upload and update(replace) embedded certificate  with password in a policy
        .\UploadCertificateWithPassword.ps1 -uemHostAndPort server01:18084 -tenantGuid SRP00000 -authType Local -username admin -password password -domain example -userGuid c4321652-24d9-46b3-803c-e5269bded459 -certPath C:\Users\username\Desktop\certificate.pfx -certPassword password -profileGuid 06cb9d26-ce92-461d-bdf9-3f1e8296c2c8
#>
Param(
    [Parameter(
        Mandatory=$true,
        HelpMessage="UEM Host and port."
    )]
    [ValidateNotNullorEmpty()]
    [string]$uemHostAndPort,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Tenant guid to authenticate with."
    )]
    [ValidateNotNullorEmpty()]
    [string]$tenantGuid,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Choose (AD | Local) authType."
    )]
    [ValidateSet('AD', 'Local')]
    [string]$authType,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Username to authenticate with."
    )]
    [ValidateNotNullorEmpty()]
    [string]$username,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Password to authenticate with."
    )]
    [ValidateNotNullorEmpty()]
    [string]$password,

    [Parameter(
        Mandatory=$false,
        HelpMessage="Domain for AD authenticate type."
    )]
    [string]$domain,

    [Parameter(
        Mandatory=$true,
        HelpMessage="User guid to get user's embedded certificate."
    )]
    [ValidateNotNullorEmpty()]
    [string]$userGuid,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Non-Dynamics certificate path."
    )]
    [ValidateNotNullorEmpty()]
    [string]$certPath,

    [Parameter(
      Mandatory=$true,
      HelpMessage="Non-Dynamics certificate password."
    )]
    [AllowEmptyString()]
    [string]$certPassword,

    [Parameter(
        Mandatory=$false,
        HelpMessage="Profile guid to set or replace an embedded certificate for a profile assigned to a user."
    )]
    [ValidateNotNullorEmpty()]
    [string]$profileGuid
)

if(-not (Test-Path $certPath))
{
    throw [System.IO.FileNotFoundException] "$certPath file not found";
}

$Global:authString = $null;
$Global:uemBaseUrl = New-Object -TypeName System.Text.StringBuilder;
$Global:fileName = Split-Path $certPath -Leaf;
$Global:tmpStartFile = New-TemporaryFile;
$Global:tmpCertContent = New-TemporaryFile;
$Global:tmpEndFile = New-TemporaryFile;
$Global:tmpBodyRequest = New-TemporaryFile;

[System.Net.ServicePointManager]::SecurityProtocol = [System.Net.SecurityProtocolType]::Tls12;

<#
    * Generate authorization header required to perform the REST call.
    *
    * @return
    *       True if successful, false otherwise.
#>
function setup()
{
    $methodName = "setup()";
    enterMethod($methodName);
    $provider = $null;

    $Global:uemBaseUrl.Append("https://");
    $Global:uemBaseUrl.Append($uemHostAndPort);
    $Global:uemBaseUrl.Append("/");
    $Global:uemBaseUrl.Append($tenantGuid);
    $Global:uemBaseUrl.Append("/api/v1");

    # Create and set the headers for the REST request
    $headers = @{}
    $headers.add("Content-Type", "application/vnd.blackberry.authorizationrequest-v1+json")

    # Create body for REST call
    $body = @{}
    $body.add("provider",$authType);
    $body.add("username", $username);
    $body.add("password", [Convert]::ToBase64String([System.Text.Encoding]::UTF8.GetBytes($password)));
    if($authType -eq "AD")
    {
        if(-not($domain))
        {
            throw [System.ArgumentNullException]"The domain argument is required when authType=AD";
        }
        $body.add("domain", $domain);
    }

    $request = New-Object -TypeName System.Text.StringBuilder;
    $request.Append($Global:uemBaseUrl);
    $request.Append("/util/authorization");

    # Invoke RESTMethod to call authorization route to generate the authorization headers
    try
    {
        $response = Invoke-RestMethod -Method POST -Headers $headers -Uri $request.ToString() -Body ($body | ConvertTo-Json);
        $Global:authString = $response
    }
    catch
    {
        handleError $_;
    }

    Write-Host([string]::Format("Authorization header string generated: {0}", $Global:authString));
    Write-Host("Setup Complete...");
    exitMethod($methodName);
}

function createRequestBody()
{
    $methodName = "createRequestBody()";
    enterMethod($methodName);

    $encodedCertPassword = [Convert]::ToBase64String([System.Text.Encoding]::UTF8.GetBytes($certPassword));
    $jsonPassword = @{"password" = $encodedCertPassword} | ConvertTo-Json;
    $byteArray = Get-Content -Raw -Path $certPath;

    Add-Content $Global:tmpStartFile "--clientDefinedContentBoundary";
    Add-Content $Global:tmpStartFile "Content-Type: application/vnd.blackberry.deviceusercertificatemetadata-v1+json`n";
    Add-Content $Global:tmpStartFile $jsonPassword;
    Add-Content $Global:tmpStartFile "--clientDefinedContentBoundary";
    Add-Content $Global:tmpStartFile "Content-Type: application/pkcs12";
    Add-Content $Global:tmpStartFile "Content-Disposition: attachment; filename=""$Global:fileName""";
    Add-Content $Global:tmpStartFile "Content-ID: clientDefinedRootPartIdentifier";

    Add-Content $Global:tmpCertContent $byteArray;

    Add-Content $Global:tmpEndFile "--clientDefinedContentBoundary--";

    Get-Content -Raw $Global:tmpStartFile, $Global:tmpCertContent, $Global:tmpEndFile | Set-Content $Global:tmpBodyRequest;

    exitMethod($methodName);
}

function createHeaders()
{
    $methodName = "createHeaders()";
    enterMethod($methodName);

    $headers = @{};
    $headers.add("Authorization", $Global:authString);
    $headers.Add("Content-type", "multipart/related; boundary=`"clientDefinedContentBoundary`"; start=`"clientDefinedRootPartIdentifier`"");

    exitMethod($methodName);

    return $headers;
}

function uploadUserCertificateWithPassword()
{
    $methodName = "uploadUserCertificateWithPassword()";
    enterMethod($methodName);

    $request = New-Object -TypeName System.Text.StringBuilder;
    $response = $null;

    #Create request body with multipart/related content type
    createRequestBody;

    # Create headers for the REST request
    $headers = createHeaders;

    $request.Append($Global:uemBaseUrl);
    $request.Append("/users/");
    $request.Append($userGuid);
    $request.Append("/embeddedCertificates");

    # Invoke RESTMethod to call API to obtain user embedded certificate in the system
    try
    {
        $response = Invoke-RestMethod -Method POST -Headers $headers -InFile $Global:tmpBodyRequest -Uri $request.ToString();
    }
    catch
    {
         handleError $_;
    }
    exitMethod($methodName);
    return $response;
}

function uploadUserProfileCertificateWithPassword()
{
    $methodName = "uploadUserProfileCertificateWithPassword()";
    enterMethod($methodName);

    $request = New-Object -TypeName System.Text.StringBuilder;
    $response = $null;

    #Create request body with multipart/related content type
    createRequestBody;

    # Create headers for the REST request
    $headers = createHeaders;

    $request.Append($Global:uemBaseUrl);
    $request.Append("/users/");
    $request.Append($userGuid);
    $request.Append("/profiles/");
    $request.Append($profileGuid);
    $request.Append("/embeddedCertificates");

    # Invoke RESTMethod to call API to obtain user embedded certificate in the system
    try
    {
        $response = Invoke-RestMethod -Method PUT -Headers $headers -InFile $Global:tmpBodyRequest -Uri $request.ToString();
    }
    catch
    {
         handleError $_;
    }

    exitMethod($methodName);
    return $response;
}

<#
    * Logging for method entry.
    *
    * @param methodName
    *       Name of the method to log
#>
function enterMethod($methodName)
{
    Write-Host([string]::Format("Entering {0}...", $methodName));
}

<#
    * Logging for method exit.
    *
    * @param methodName
    *       Name of the method to log.
#>
function exitMethod($methodName)
{
    Write-Host([string]::Format("Exiting {0}...", $methodName));
}

<#
    * Handle any errors that occur when performing the REST call.
    *
    * @param error
    *       The group create context.
#>
function handleError($error)
{
    Write-Error ("The command cound not be completed.  `r`nReason: " + $error.exception.message + " `r`nDetails: "+ $error.errordetails);
    exit(1);
}

function main()
{
    $methodName = "main()";
    $response = $null;
    enterMethod($methodName);
    try
    {
        if (setup)
        {
            if([string]::IsNullOrEmpty($profileGuid))
            {
               $response = uploadUserCertificateWithPassword;
            }
            else
            {
                $response = uploadUserProfileCertificateWithPassword;
            }
        }
    }
    catch
    {
        handleError $_;
    }
    Write-Host($response | Select-Object -Last 1 | ConvertTo-Json -Depth 4);

    exitMethod($methodName);
    Write-Host("Complete!");

    Remove-Item -Path $Global:tmpStartFile;
    Remove-Item -Path $Global:tmpCertContent;
    Remove-Item -Path $Global:tmpEndFile;
    Remove-Item -Path $Global:tmpBodyRequest;

    exit(0);
}

# Call main function.
main;